/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.StyledBox");

dojo.require("jazz.ui.internal.ImgMenuItem");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("dojo.fx");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("jazz.ui", "StyledBoxMessages");

(function() {
var ImgMenuItem = jazz.ui.internal.ImgMenuItem;
var TrimStyles = jazz.ui.styledBox.TrimStyles;
var ResizeNotifier = jazz.util.ResizeNotifier;
var messages;
var preloaded = false;

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.StyledBox",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("jazz.ui", "templates/StyledBox.html"),
	
	/*
	 * Defaults: Pass any of these properties into the constructor to set them. 
	 */
	
	contentNode: null,  	//DOM node for contents of dialog
	"class": null,			//specify this value to add custom classname to box wrapper div
	width: "",				//Optional - zero means natural width
	height: "",				//Optional - zero means natural height
	primaryTitle: "",
	secondaryTitle: "",
	truncateTitle: true,	//truncates the primary title to fit the box, and adds a ... to the end.
	trim: TrimStyles.TRIMLESS,
	headerHoverState: true,				//causes viewlet to change its rendering on mouse over (adds -hover class to box's root node).
	headerDefaultToHoverState: false,  	//causes box to default to its hover state rendering (if true, headerHoverState value is ignored)
	showBackground: false,
	iconPath: "",
	menuItems: null,  	//array of objects in the form {label (string), iconClass (string), onClick (function), index (int, optional)}
	closable: true,
	collapsible: false,
	collapsed: false,
	headerless: false,	//hides the header title, icon and bgcolor. Header buttons appear if defined.  No effect on TRIMLESS. Ignores collapsible.
	destroyContents: false, //causes the box to destroy the widget passed in 'contentNode' on close (if it's a widget or the domNode for a widget)

	//callback functions
	onClose: null,		//in your handlers, return boolean false to veto action (close, expand, collapse).
	onOpen: null,
	onCollapse: null,
	onExpand: null,

	constructor: function() {
		this._messages = messages = dojo.i18n.getLocalization("jazz.ui", "StyledBoxMessages");
		if (!preloaded) {
			setTimeout(_preloadSecondaryImages,2000);
			preloaded = true;
		}
	},
	postCreate: function() {
		this.setWidth();
		this.setHeight();
		
		if (!this.contentNode) this.contentNode = document.createElement("div");
		this._bodyContainer.appendChild(this.contentNode);
		
		this.setTitle(this.primaryTitle);
		this.setSecondaryTitle(this.secondaryTitle);
		
		this.setTrim(this.trim);
		this.setBackground(this.showBackground);
		if (this.menuItems) {
			this._toggleMenu.style.display = "inline";
			if (!this._openMenuEvent) {
				this._openMenuEvent = this.connect(this._toggleMenu, 'onclick', this._openMenu);
			}
		}

		this.setIcon();
	
		this.setClosable(this.closable);
		
		if (this.collapsible && !this.headerless) {
			this._collapse.style.display = "inline";
			this.connect(this._collapse,'onclick',this._toggleCollapse);
			this.connect(this._header,'ondblclick',this._toggleCollapse);
			this.connect(this._headerButtons,'ondblclick',dojo.stopEvent);
			
			var args = {node: this._middleSection, duration: 200};
			this._expandAnimation = dojo.fx.wipeIn(args);
			this._collapseAnimation = dojo.fx.wipeOut(args);
			this.connect(this._expandAnimation, "onEnd", this._onCollapseAnimationEnd);
			this.connect(this._collapseAnimation, "onEnd", this._onCollapseAnimationEnd);
			this._updateCollapsedState();
		}
		this._menuImg.src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/down-arrow.gif");
		this._closeImg.src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/close.gif");
		if (this.onOpen && typeof this.onOpen == "function") this.onOpen();
	},
	_setupTruncate: function() {
		if (this.primaryTitle && !this.headerless && this.truncateTitle) {
			this._queueTruncate(true);
			if (!this._windowResizeEvent) {
				this._windowResizeEvent = ResizeNotifier.register({
					context: this,
					callback: this._queueTruncate
				});	
			}
		}
	},
	setClosable: function(closable) {
		this.disconnect(this._closeEvent);
		if (closable) {
			this._close.style.display = "inline";
			this._closeEvent = this.connect(this._close, 'onclick', this.close);
		} else {
			this._close.style.display = "none";
		}
		this.closable = closable;
	},
	_toggleCollapse: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (this.collapsed) {
			//call onExpand callback, if exists
			if (this.onExpand) {
				try {
					var keepGoing = this.onExpand();
					if (keepGoing === false) return;	
				} catch(e) { 
					//eat the error 
				}
			}
			dojo.removeClass(this._middleSection,"middle-section-collapsed");
			dojo.removeClass(this._header,"header-collapsed");
			this._middleSection.style.height = "1px";
			this._expandAnimation.play();	
		} else {
			//call onCollapse callback, if exists
			if (this.onCollapse) {
				try {
					var keepGoing = this.onCollapse();
					if (keepGoing === false) return;	
				} catch(e) { 
					//eat the error 
				}
			}
			this._collapseAnimation.play();
		}		
	},
	_onCollapseAnimationEnd: function(){
		this.collapsed = !this.collapsed;
		this._updateCollapsedState();
		this._middleSection.style.overflow = "";
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
	_updateCollapsedState: function(){
		if (this.collapsed) {
			dojo.addClass(this._middleSection,"middle-section-collapsed");
			dojo.addClass(this._header,"header-collapsed");
			this._collapse.className = "expand";
			this._collapse.title = messages.expand;
			this._collapseImg.src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/expand.gif");
			this._collapseImg.alt = messages.expand;
		} else {
			this._collapse.className = "collapse";
			this._collapse.title = messages.collapse;
			this._collapseImg.src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/minimize.gif");
			this._collapseImg.alt = messages.collapse;
		}
	},
	addMenuItem: function(args) {
		// args format: {/* string */ label, /* function */ onClick, /* string */ iconClass, /* int (optional)*/ index}
		if (!this.menuItems) {
			this.menuItems = new Array();
			this._toggleMenu.style.display = "inline";
			if (!this._openMenuEvent) {
				this._openMenuEvent = this.connect(this._toggleMenu, 'onclick', this._openMenu);
			}
		}
		if (typeof args.index == "number") {
			this.menuItems.splice(args.index,0,args);
		} else {
			this.menuItems.push(args);	
		}
		// if the menu has already been created, insert it
		if (this._menu) {
			if (typeof args.index == "number") {
				this._menu.addChild(new ImgMenuItem(args), args.index);	
			} else {
				this._menu.addChild(new ImgMenuItem(args));
			}
		}
	},
	/*
	 * Closes the box and removes it from the DOM.
	 * Called automatically when the user clicks on the box's close button.
	 */
	close: function(event) {
		if (event) dojo.stopEvent(event);
		
		// call onClose callback function, if it exists
		if (this.onClose && typeof this.onClose == "function") {
			var keepClosing = this.onClose();
			if (keepClosing === false) {
				return false;
			}
		}
		
		if (dojo.isIE == 7) {
			//IE7 does not fadeOut alpha PNG drop shadows properly (shows ugly black bars) - so just close it
			this._onClose();
		} else {
			//fadeOut
			var args = {node: this.domNode, duration: 200};
			var fadeOut = this._fadeOutAnim = dojo.fadeOut(args);
			this.connect(fadeOut, "onEnd", this._onClose);
			fadeOut.play();
			this._fadeOutTimeout = setTimeout(dojo.hitch(this,this._onClose),1200);
		}
	},
	setIcon: function(iconPath) {
		if (!iconPath) iconPath = this.iconPath;
		if (!iconPath || this.headerless) {
			if (this._iconImage) {
				this._iconImage.parentNode.removeChild(this._iconImage);
				this._headerIcon.style.display = "";
				this._iconImage = null;
			}
			return;
		}
		if (this._iconImage) {
			this._iconImage.src = iconPath;
		} else {
			this._headerIcon.style.display = "block";
			var iconImage = document.createElement("img");
			iconImage.src = iconPath;
			iconImage.className = "icon-image";
			this._headerIcon.appendChild(iconImage);
			this._iconImage = iconImage;
		}
		this.iconPath = iconPath;
	},
	setHeight: function(/*String with unit*/ height) {
		if (height) {
			this._bodyContainer.style.height = height;
			this.height = height;
		} else if (this.height) {
			this._bodyContainer.style.height = this.height;
		} else {
			this._bodyContainer.style.height = "";
		}
	},
	setWidth: function(/*String with unit*/ width){
		if (width) {
			this.domNode.style.width = width;
			this.width = width;
		} else if (this.width){
			this.domNode.style.width = this.width;
		} else {
			this.domNode.style.width = "";
		}
	},
	setTitle: function(/* string */ title) {
		if (this._headerPrimary.firstChild) {
			this._headerPrimary.replaceChild(document.createTextNode(title),this._headerPrimary.firstChild);	
		} else {
			this._headerPrimary.appendChild(document.createTextNode(title));
		}
		this.primaryTitle = title;
		this._setupTruncate();
	},
	setSecondaryTitle: function(/* string */ title) {
		this._headerSecondary.innerHTML = title;
		this.secondaryTitle = title;
		this._setupTruncate();
	},
	setTrim: function(trim) {
		this.disconnect(this._addHoverHandle);
		this.disconnect(this._removeHoverHandle);
		if (this.trim == TrimStyles.TRIMLESS && trim != TrimStyles.TRIMLESS) {
			this.disconnect(this._headerLinkHandle);
		}
		dojo.removeClass(this.domNode,this.trim);
		dojo.removeClass(this.domNode,this.trim+"-hover");
		dojo.removeClass(this.domNode,"jazz-ui-StyledBox-hover");
		dojo.removeClass(this.domNode,TrimStyles.TRIMLESS+"-showheader");
		dojo.addClass(this.domNode,trim);
		if (this.headerless && trim != TrimStyles.TRIMLESS) {
			dojo.addClass(this.domNode,"jazz-ui-StyledBox-noheader");
		} else {
			dojo.removeClass(this.domNode,"jazz-ui-StyledBox-noheader");
		}
		if (trim == TrimStyles.TRIMLESS) {
			this._headerLinkHandle = this.connect(this._headerLink,'onclick', this._toggleTrimlessHeader);
			this._addHoverHandle = this.connect(this.domNode,'onmouseover',this._addHoverClass);
			this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);
			this._headerVisible = false;
			if (this.collapsed)	this._toggleTrimlessHeader();
		} else if ((this.headerHoverState || this.headerless) && !this.headerDefaultToHoverState) {
			//if we want the header to change style on hover
			var node;
			if (this.headerless) 
				node = this.domNode;
			else 
				node = this._header; 
			this._addHoverHandle = this.connect(node,'onmouseover',this._addHoverClass);
			this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);			
		} else if (this.headerDefaultToHoverState) {
			dojo.addClass(this.domNode,trim+"-hover");
			dojo.addClass(this.domNode,"jazz-ui-StyledBox-hover");
		}
		this.trim = trim;
	},
	setBackground: function(background) {
		if (background)
			dojo.addClass(this._contentWrapper,"showBackground");
		else
			dojo.removeClass(this._contentWrapper,"showBackground");
	},
	destroy: function() {
		ResizeNotifier.unregister(this._windowResizeEvent);
		if (this.truncateTimeout) {
			// cancel deferred title truncation
			clearTimeout(this.truncateTimeout);
			delete this.truncateTimeout;
		} 
		if (this._menu) {
			this._menu.destroy();
			delete this._menu;
		}
		this.inherited(arguments);
	},
	_toggleTrimlessHeader: function(event, kill) {
		if (event)
			event.preventDefault();
			
		if (this._headerVisible || kill) {
			dojo.removeClass(this.domNode,TrimStyles.TRIMLESS+"-showheader");
			this._headerLink.innerHTML = messages.showHeader;
			this._headerLink.title = messages.showHeader;
			this._headerVisible = false;
		} else {
			dojo.addClass(this.domNode,TrimStyles.TRIMLESS+"-showheader");
			this._headerLink.innerHTML = messages.hideHeader;
			this._headerLink.title = messages.hideHeader;
			this._headerVisible = true;
		}
		this._headerLink.blur();
	},
	_createMenu: function (){
		if (this._menu) {
			this._menu.destroy();
		}
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode, "jazz-ui-StyledBox-menu");
		// add known menu contributions
		if (this.menuItems) {
			for (var i=0;i<this.menuItems.length;i++) {
				if (this.menuItems[i].separator) {
					menu.addChild(new dijit.MenuSeparator());
					continue;
				}
				if (!this.menuItems[i].label) {
					console.error("Error creating menu:  Menu item at index "+i+" is missing a label.");
					continue;
				}
				menu.addChild(new ImgMenuItem(this.menuItems[i]));
			}
		}		
		this._menu = menu;
	},
	_openMenu: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		//calculate the position of the drop down menu
		var pos = dojo.coords(this._toggleMenu);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		
		//Display menu
		if (!this._menu) {
			this._createMenu();
		}
		dijit.popup.open({ popup: this._menu, parent: this, around: this._toggleMenu, orient: {"BR":"TR"}, onExecute: dojo.hitch(this,this._closeMenu), onCancel: dojo.hitch(this,this._closeMenu)});
		this._menu.focus();
		//register event handlers that cause menu to close
		if(!this._menuCloseOnBody){
			this._menuCloseOnBody = this.connect(document.body, "onclick", this._closeMenu);	
		}
		if (!this._menuCloseOnViewlet) {
			this._menuCloseOnViewlet = this.connect(this.domNode,"onmouseup", this._closeMenu);	
		}
		//style menu icon to indicate open state
		dojo.addClass(this._toggleMenu,"open-menu");
		this._addHoverClass(null, true);
	},
	_closeMenu: function(event) {
		this.disconnect(this._menuCloseOnBody);
		this.disconnect(this._menuCloseOnViewlet);
		this._menuCloseOnBody = null;
		this._menuCloseOnViewlet = null;
		dojo.removeClass(this._toggleMenu,"open-menu");
		dijit.popup.close(this._menu);
		this._removeHoverClass(null, true);
		this._toggleMenu.focus();
	},
	_addHoverClass: function (event, force) {
		dojo.addClass(this.domNode,this.trim+"-hover");
		dojo.addClass(this.domNode,"jazz-ui-StyledBox-hover");
		if (force) {
			this.disconnect(this._removeHoverHandle);
			this._removeHoverHandle = null;
		} 
	},
	_removeHoverClass: function (event, force) {
		dojo.removeClass(this.domNode,this.trim+"-hover");
		dojo.removeClass(this.domNode,"jazz-ui-StyledBox-hover");
		if (force) {
			if (!this._removeHoverHandle) {
				this._removeHoverHandle = this.connect(this.domNode,'onmouseout',this._removeHoverClass);
			}
		} 
	},
	/*
	 * Handler for onresize truncate call.
	 * Use yielding so that UI is not hung in IE6/7 during truncate's while loop.
	 */ 
	_queueTruncate: function(force) {
		if (!this.truncateTimeout) {
			this.truncateTimeout = setTimeout(dojo.hitch(this,this._truncate,force),400);
		}
	},
	_truncate: function(force) {
		if (!this.primaryTitle || !this.truncateTitle) return;
		this.truncateTimeout = null;
		var node = this._headerPrimary;
		// force is used when the title has been changed and the truncate algorithm needs to be run
		// even if the clientWidth is the same
		// force == true is used since this function is called from many places, some of which
		// pass in parameters
		if (force == true || (this._titleIsTruncated && this._header.clientWidth > this._prevWidth)) {
			node.innerHTML = this.primaryTitle;
			this._titleIsTruncated = false;
			node.title = "";
		}
		if (this._headerText.clientHeight > this._header.clientHeight + 5) {
			var title = node.innerHTML;
			while (this._headerText.clientHeight > this._header.clientHeight + 5) {
				if (this._headerText.clientHeight > this._header.clientHeight * 2) {
					title = title.substring(0,title.length/2);
				} else {
					title = title.substring(0,title.length-2);
				}
				while (title.lastIndexOf(" ") == title.length-1)
					title = title.substring(0,title.length-1);
				node.innerHTML = title + "...";
				if (node.innerHTML.length <= 6) break;
			}
			node.title = this.primaryTitle;
			this._titleIsTruncated = true;
			this._prevWidth = this._header.clientWidth;
		}
	},
	_onClose: function() {
		clearTimeout(this._fadeOutTimeout);
		this._fadeOutTimeout = null;
		if (this._fadeOutAnim) {
			this._fadeOutAnim.stop();
			this._fadeOutAnim = null;
		}
		if (this.domNode && this.domNode.parentNode) {
			this.domNode.parentNode.removeChild(this.domNode);	
		}
		if (this.destroyContents) {
			// Destroy the content node widget for the dialog
			var widget = dijit.byId(this.contentNode.id);
			if (widget && dojo.isFunction(widget.destroyRecursive)) {
			    widget.destroyRecursive()
			}
		}
		// Now destroy the dialog itself
		this.destroyRecursive();
	}
});
var pictures = new Array();
function _preloadSecondaryImages(){
	var versionId = (dojo.dependency !== undefined) ? "/_ver"+dojo.dependency.getWebBundleVersion("net.jazz.web.ui") : "/_ver";
	pictures.push((new Image()).src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/close-hov.gif"+versionId));
	pictures.push((new Image()).src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/expand-hov.gif"+versionId));
	pictures.push((new Image()).src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/minimize-hov.gif"+versionId));
	pictures.push((new Image()).src = dojo.moduleUrl("jazz.ui", "graphics/StyledBox/down-arrow-hov.gif"+versionId));
}
})();