/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");
dojo.require("com.ibm.team.repository.web.client.internal.licensing"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");

var messages;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var checkExpiringCALs = com.ibm.team.repository.web.client.internal.licensing.checkExpiringCALs;
var getExpirationInfo = com.ibm.team.repository.web.client.internal.licensing.getExpirationInfo;
var getLicenseInfo = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;

var LARGE_NUMBER = 999999;

dojo.declare("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/LicenseInfoWidget.html"),
	
	postCreate: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");
		
		dojo.subscribe(com.ibm.team.repository.web.internal.events.licenseChange, this, "_showLicenseInfo");
		
		var self = this;
		setTimeout(function() {
			getLicenseInfo(new ServiceResponseHandler(self, "_showLicenseInfo", "_showLicenseInfoErr"));
		}, 4000);		
	},
	_showLicenseInfo: function(licenseInfo) {
		var disabledReason = dojo.getObject("serverLicense.disabledReason", false, licenseInfo || "");
		var expirationTime = dojo.getObject("serverLicense.expirationTime", false, licenseInfo || "");
		
		//Find out if we have expiring CALs
		var minDaysRemaining = LARGE_NUMBER;
		var buyURL = ""; 
		var anyExpiringCALS = false;
		var types = licenseInfo.types || [];
		if (licenseInfo && licenseInfo.floatingSetupStatus) {
			var floatingTypes = licenseInfo.floatingLicenseTypes || [];
        	var combined = types.concat(floatingTypes);
		} else {
			combined = types;
		}
		dojo.some(combined, function(type){
			if (type.expirationTime != null) {
				anyExpiringCALS = true;
				//Find out how many days till it expires
				var expirationInfo = getExpirationInfo(type.expirationTime);
				if (expirationInfo.isExpired) {
					minDaysRemaining = -1;
					buyURL = type.buyURL;
					return;
				} else {
					var days = expirationInfo.daysRemaining;
					if(days < minDaysRemaining ) {
						minDaysRemaining = days;
						buyURL = type.buyURL;
					}
				}
			}
		});
		
		// no expiration time for the server license
		if (!expirationTime && !disabledReason) {
			this.expirationInfo.innerHTML = "";
			this.expirationWrapper.style.display = "none";
			
			//check to see if any CALs are expiring
			if (anyExpiringCALS) {
				this._displayExpiringCAL(minDaysRemaining,buyURL);
			}
			
		} else if(disabledReason) {
			this.expirationInfo.className = "expirationWarning";
			this.expirationInfo.innerHTML = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>" + disabledReason + "</a>";
		} else {
			var expirationInfo = getExpirationInfo(expirationTime);

			if (anyExpiringCALS && expirationInfo.daysRemaining > minDaysRemaining && !expirationInfo.isExpired) {
				//a CAL expires first
				this._displayExpiringCAL(minDaysRemaining,buyURL);
			}
			else {
				//server expires first
				var purchaseLink = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>";
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.trialLicense, [purchaseLink, "</a> "]);
				if (expirationInfo.isExpired || expirationInfo.daysRemaining <= 14) {
					this.expirationInfo.className = "expirationWarning";
				}
				else {
					this.expirationInfo.className = "expirationInfo";
				}
				if (expirationInfo.isExpired) {
					//server has expired. see if CAL is also expired
					if(minDaysRemaining == -1) {
						//cal also expired. display comined message
						var calPurchaseLink = "<a href='" + buyURL + "' target='_blank'>";	
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.bothExpired, [ purchaseLink, "</a> ", calPurchaseLink, "</a> "]);
					} else {
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [this.expirationInfo.innerHTML]);	
					}
				}
				else 
					if (expirationInfo.daysRemaining === 0) {
						this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [this.expirationInfo.innerHTML]);
					}
					else 
						if (expirationInfo.daysRemaining === 1) {
							this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [this.expirationInfo.innerHTML]);
						}
						else {
							var daysLeft = expirationInfo.daysRemaining;
							this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [this.expirationInfo.innerHTML, daysLeft]);
						}
				this.expirationWrapper.style.display = "inline";
			}
		}
		
	},
	_showLicenseInfoErr: function(errorObj) {
		console.warn(dojo.string.substitute(this.messages.noLicenseInfo, [ errorObj.message ]));
	},
	
	_displayExpiringCAL: function(minDaysRemaining,buyURL) {
		var purchaseLink = "<a href='" + buyURL + "' target='_blank'>";	
		this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.calLicense, [ purchaseLink, "</a> "]);
		this.expirationInfo.className = (minDaysRemaining < 14) ? "expirationWarning" : "expirationInfo";
				
		if (minDaysRemaining === -1) { //expired
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [this.expirationInfo.innerHTML]);
		} else if (minDaysRemaining === 0) {
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [ this.expirationInfo.innerHTML ]);
		} else if (minDaysRemaining === 1) {
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [ this.expirationInfo.innerHTML ]);
		} else {
			var daysLeft = minDaysRemaining;
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [ this.expirationInfo.innerHTML, daysLeft ]);
		}
		this.expirationWrapper.style.display = "inline";
	}
	
});

var EVENTS_ROOT = "com.ibm.team.repository.web.internal.events";
dojo.setObject(EVENTS_ROOT, {
	licenseChange: EVENTS_ROOT + ".licensesChanged"
});

})();