/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer");

/*
 * Imports 
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab");

(function() {

/*
 * Aliases 
 */
var SimpleTab = com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab;

/*
 * Class 
 */
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	callbackTabSwitch: null,

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/SimpleTabContainer.html"),
	
	/*
	 * Static Variable Container
	 */
	_static: function() {}, 	

	// Public Methods //------------------------------------------------------->

	/*
 	 * @param Object args
 	 * 
 	 * @return 
 	 */
	constructor: function(args) {
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this._initializeStaticVariables();
		this._initializeInstanceVariables();
		this._initializeWidget();
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		// Call Back Tab Switch
		// Do not delete (do not own)
		
		// Tabs
		if (this._tabHolder) {
			for (var tabID in this._tabHolder) {
				if (this._tabHolder[tabID]) {
					this._tabHolder[tabID].destroy();
					delete this._tabHolder[tabID];
				}
			}
			delete this._tabHolder;
		}

		// Selected Tab
		// No need to delete (selected tab stored in tabHolder)

		this.inherited(arguments);
	},

	/*
 	 * @param String tabID
 	 * @param String title
 	 * @param Function callbackGetNode
 	 * 
 	 * @return void
 	 */		
	addTab: function(tabID, title, callbackGetNode) {
		// Create new tab
		var tabWidget = new SimpleTab({
			tabID: tabID,
			tabTitle: title,
			callbackGetNode: callbackGetNode,
			callbackOnClick: dojo.hitch(this, this._handleEventOnClickTab)
		});
		// Add tab to container
		this._tabArea.appendChild(tabWidget.domNode);
		// Store tab
		this._tabHolder[tabID] = tabWidget;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return void
 	 */		
	selectTab: function(tabID) {
		
		if ((tabID in this._tabHolder) === false) {
			return;
		}
		
		var previousTab = this._selectedTab;
		
		if (previousTab && 
			(previousTab.tabID === tabID)) {
			// Same tab
			// Nothing to do
			return;
		} else if (previousTab) {
			// Deselect previous tab
			previousTab.deselect();
		}

		var currentTab = this._tabHolder[tabID];
		
		// Select current tab
		currentTab.select();
		
		// Update selection
		this._selectedTab = currentTab;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return Element
 	 */		
	getContentNode: function(tabID) {
		if ((tabID in this._tabHolder) === false) {
			return null;
		}
		return this._tabHolder[tabID].contentNode;
	},

	// Private Methods //------------------------------------------------------>

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeInstanceVariables: function() {
		this._tabHolder = new Object();
		this._selectedTab = null;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initializeWidget: function() {
	},
	
	/*
 	 * @param SimpleTab tab
 	 * 
 	 * @return void
 	 */		
	_handleEventOnClickTab: function(tab) {
		
		var newTabID = tab.tabID;

		var oldTabID = null;
		if (this._selectedTab) {
			oldTabID = this._selectedTab.tabID;
		}
		
		if (oldTabID === newTabID) {
			// Same tab
			// Nothing to do
			return;
		}
		
		// Update tab selection
		this.selectTab(newTabID);
		// Fire event
		if (this.callbackTabSwitch) {
			this.callbackTabSwitch(newTabID, oldTabID);
		}		

	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_noop: function() {
	}		
	
});

})();
